import React, { useState, useMemo, forwardRef, useRef, useEffect } from 'react';
import noop$1 from 'lodash/noop';
import { useAutoPositioner, useForkRef } from '@highlight-ui/utils-hooks';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import classnames from 'classnames';
import toPairs from 'lodash/toPairs';
import groupBy from 'lodash/groupBy';
import { Typography } from '@highlight-ui/typography';
import noop from 'lodash/fp/noop';
import { Icon } from '@highlight-ui/icon';
import { Checkbox } from '@highlight-ui/checkbox';
import { Avatar } from '@highlight-ui/avatar';
import uniqueId from 'lodash/uniqueId';
import { Button } from '@highlight-ui/button';
import { ColorPicker } from '@highlight-ui/color-picker';
import { Input } from '@highlight-ui/input';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

var build = {exports: {}};

(function (module, exports) {
(function(a,b){module.exports=b();})('undefined'==typeof self?commonjsGlobal:self,function(){return function(a){function b(d){if(c[d])return c[d].exports;var e=c[d]={i:d,l:!1,exports:{}};return a[d].call(e.exports,e,e.exports,b),e.l=!0,e.exports}var c={};return b.m=a,b.c=c,b.d=function(a,c,d){b.o(a,c)||Object.defineProperty(a,c,{configurable:!1,enumerable:!0,get:d});},b.n=function(a){var c=a&&a.__esModule?function(){return a['default']}:function(){return a};return b.d(c,'a',c),c},b.o=function(a,b){return Object.prototype.hasOwnProperty.call(a,b)},b.p='./',b(b.s=2)}([function(a,b,c){Object.defineProperty(b,'__esModule',{value:!0}),b.useClickOutside=function(){function a(a){b&&b.current&&(b.current.contains(a.target)?h({hasClickedOutside:!1}):h({hasClickedOutside:!0}));}const b=Object(d.useRef)();var c=Object(d.useState)({hasClickedOutside:!1}),e=f(c,2);const g=e[0],h=e[1];return Object(d.useEffect)(()=>(window.PointerEvent?document.addEventListener('pointerdown',a):(document.addEventListener('mousedown',a),document.addEventListener('touchstart',a)),()=>{window.PointerEvent?document.addEventListener('pointerdown',a):(document.removeEventListener('mousedown',a),document.removeEventListener('touchstart',a));}),[]),[b,g.hasClickedOutside]};var d=c(1);c.n(d);var f=function(){function a(a,b){var c=[],d=!0,e=!1,f=void 0;try{for(var g,h=a[Symbol.iterator]();!(d=(g=h.next()).done)&&(c.push(g.value),!(b&&c.length===b));d=!0);}catch(a){e=!0,f=a;}finally{try{!d&&h['return']&&h['return']();}finally{if(e)throw f}}return c}return function(b,c){if(Array.isArray(b))return b;if(Symbol.iterator in Object(b))return a(b,c);throw new TypeError('Invalid attempt to destructure non-iterable instance')}}();},function(a){a.exports=React;},function(a,b,c){a.exports=c(3);},function(a,b,c){Object.defineProperty(b,'__esModule',{value:!0});var d=c(0);c.d(b,'useClickOutside',function(){return d.useClickOutside});}])});

}(build));

function getSelectedValues(_a) {
    var multiple = _a.multiple, option = _a.option, selectedOptions = _a.selectedOptions, selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly;
    if (!multiple) {
        return [option];
    }
    if (selectedOptionsWithValuesOnly.includes(option.value)) {
        return selectedOptions.filter(function (selectedOption) { return selectedOption.value !== option.value; });
    }
    return __spreadArray(__spreadArray([], selectedOptions, true), [option], false);
}
/**
 * Returns options that are pre-selected and disabled.
 * @param allOptions
 * @param preSelectedOptions
 */
function getFixedOptions(allOptions, preSelectedOptions) {
    var allDisabled = allOptions.filter(function (option) { return option.disabled; });
    return allDisabled.filter(function (disabledOption) {
        return preSelectedOptions.some(function (option) { return option.value === disabledOption.value; });
    });
}
/**
 * Returns a list of option that are fixed and are enabled
 * @param options
 * @param preSelectedOptions
 */
function getAllSelectableOptions(options, preSelectedOptions) {
    var allFixed = getFixedOptions(options, preSelectedOptions);
    var allEnabled = options.filter(function (option) { return !option.disabled; });
    return __spreadArray(__spreadArray([], allFixed, true), allEnabled, true);
}

var styles$d = {"label":"List-module__3eBtAqWo__v6-1-18","labelWrapper":"List-module__2N1Czcx___v6-1-18","optionIcon":"List-module__20whLBmg__v6-1-18","option":"List-module__WP03aTyu__v6-1-18","optionCheckbox":"List-module__1xCWbakz__v6-1-18","multiLineOption":"List-module__2XiLBsmo__v6-1-18","selectedText":"List-module__3JZZHD7o__v6-1-18","disabled":"List-module__2wCQU5ux__v6-1-18","multiLineIcon":"List-module__1vZ64tez__v6-1-18","multiLineCheckbox":"List-module__37RkH873__v6-1-18","subLabel":"List-module__1Nm6OsGP__v6-1-18","list":"List-module__2ObHKkci__v6-1-18","multiLineList":"List-module__1qnvYofW__v6-1-18","fullWidth":"List-module__3axRkzKW__v6-1-18","noResults":"List-module__n88tLk_9__v6-1-18"};

var styles$c = {"labelWrapper":"Option-module__1--7xK7L__v6-1-18","optionIcon":"Option-module__peJcMBH7__v6-1-18","option":"Option-module__2ym46zUu__v6-1-18","optionCheckbox":"Option-module__zaLdwsO3__v6-1-18","multiLineOption":"Option-module__1qplZ2cB__v6-1-18","selectedText":"Option-module__3WqFb6sf__v6-1-18","disabled":"Option-module__2dJafLVp__v6-1-18","multiLineIcon":"Option-module__1CYZ9njy__v6-1-18","multiLineCheckbox":"Option-module__2pPtBn6k__v6-1-18"};

// eslint-disable-next-line import/prefer-default-export
var searchSubstring = function (string, substring) {
    return string.toLowerCase().indexOf(substring.toLowerCase());
};

var TextWithHighlight = function (_a) {
    var highlightedText = _a.highlightedText, text = _a.text;
    var startIndex = searchSubstring(text, highlightedText);
    if (startIndex === -1)
        return React.createElement(React.Fragment, null, text);
    var endIndex = startIndex + highlightedText.length;
    return (React.createElement(Typography, { component: "span", variant: "body-1" },
        text.substring(0, startIndex),
        React.createElement("b", null, text.substring(startIndex, endIndex)),
        text.substring(endIndex)));
};

function SingleLineOptionLabel(_a) {
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label }));
}

var styles$b = {"label":"MultiLineOptionLabel-module__3fgNK2Xm__v6-1-18","subLabel":"MultiLineOptionLabel-module__1iJeXNPV__v6-1-18","disabled":"MultiLineOptionLabel-module__2ftMukhR__v6-1-18"};

function MultiLineOptionLabel(_a) {
    var _b;
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement(React.Fragment, null,
        React.createElement("span", { className: styles$b.label },
            React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label })),
        React.createElement(Typography, { className: classnames(styles$b.subLabel, (_b = {},
                _b[styles$b.disabled] = option.disabled,
                _b)), component: "span", variant: "body-1", title: option.subLabel }, option.subLabel)));
}

var styles$a = {"colorIcon":"ColoredOptionLabel-module__12fSpv1G__v6-1-18"};

function ColoredOptionLabel(_a) {
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement(React.Fragment, null,
        React.createElement("span", { className: styles$a.colorIcon, style: { color: !option.disabled ? option.color : undefined } },
            React.createElement(Icon, { name: "circle", type: "solid", size: 12 })),
        React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label })));
}

var styles$9 = {"label":"AvatarOptionLabel-module__3XTEg39V__v6-1-18","subLabel":"AvatarOptionLabel-module__SE9NqSW9__v6-1-18","wrapper":"AvatarOptionLabel-module__3MbElNjg__v6-1-18","avatar":"AvatarOptionLabel-module__duV8vrCr__v6-1-18","disabledAvatar":"AvatarOptionLabel-module__3H7pKms-__v6-1-18","labelsWrapper":"AvatarOptionLabel-module__1iX_lfeO__v6-1-18","disabled":"AvatarOptionLabel-module__BohvL_H4__v6-1-18"};

function AvatarOptionLabel(_a) {
    var _b, _c;
    var option = _a.option, highlightedText = _a.highlightedText;
    return (React.createElement("div", { className: styles$9.wrapper },
        React.createElement(Avatar, { src: option.avatar, name: option.label, containerClassname: classnames(styles$9.avatar, (_b = {},
                _b[styles$9.disabledAvatar] = option.disabled,
                _b)) }),
        React.createElement("div", { className: styles$9.labelsWrapper },
            React.createElement("span", { className: styles$9.label },
                React.createElement(TextWithHighlight, { highlightedText: highlightedText, text: option.label })),
            React.createElement(Typography, { className: classnames(styles$9.subLabel, (_c = {},
                    _c[styles$9.disabled] = option.disabled,
                    _c)), component: "span", variant: "body-1", title: option.subLabel }, option.subLabel))));
}

var optionLabelComponents = {
    'single-line': SingleLineOptionLabel,
    'multi-line': MultiLineOptionLabel,
    colored: ColoredOptionLabel,
    avatar: AvatarOptionLabel,
};
function Option(_a) {
    var _b, _c, _d;
    var disabled = _a.disabled, highlightedText = _a.highlightedText, onSelect = _a.onSelect, option = _a.option, optionStyle = _a.optionStyle, optionType = _a.optionType, selected = _a.selected, className = _a.className;
    return (React.createElement("li", __assign({ "aria-selected": selected, className: classnames(styles$c.option, className, (_b = {},
            _b[styles$c.selectedText] = selected && optionStyle === 'checkmark',
            _b[styles$c.multiLineOption] = optionType === 'multi-line',
            _b[styles$c.disabled] = disabled,
            _b)), onClick: !disabled ? onSelect : noop, onKeyDown: !disabled ? onSelect : noop, role: "option", title: option.label, tabIndex: -1 }, parseMetadata(option.metadata)),
        optionStyle === 'checkbox' ? (React.createElement(Checkbox, { className: classnames(styles$c.optionCheckbox, (_c = {},
                _c[styles$c.multiLineCheckbox] = optionType === 'multi-line',
                _c)), checked: selected, readOnly: true, disabled: disabled })) : (React.createElement(Icon, { className: classnames(styles$c.optionIcon, (_d = {},
                _d[styles$c.multiLineIcon] = optionType === 'multi-line',
                _d)), name: "check", size: 14, type: "solid" })),
        React.createElement("div", { className: styles$c.labelWrapper }, React.createElement(optionLabelComponents[optionType], {
            option: option,
            highlightedText: highlightedText,
        }))));
}

var styles$8 = {"optionGroup":"OptionGroup-module__3LhwaP6p__v6-1-18","optionGroupName":"OptionGroup-module__2ZREs1EZ__v6-1-18"};

function OptionGroup(_a) {
    var name = _a.name, children = _a.children;
    return (React.createElement("li", { className: styles$8.optionGroup },
        React.createElement(Typography, { className: styles$8.optionGroupName, variant: "subtitle-2", component: "div" }, name),
        React.createElement("ul", null, children)));
}

var styles$7 = {"wrapper":"SearchInput-module__2ZvRInZD__v6-1-18","icon":"SearchInput-module__13jt8OmH__v6-1-18"};

var SearchInputWithRef = function (_a, ref) {
    var _b = _a.placeholder, placeholder = _b === void 0 ? '' : _b, value = _a.value, onTextChange = _a.onTextChange;
    var id = useState(function () { return uniqueId('component-'); })[0];
    var handleChange = function (event) {
        return onTextChange(event.currentTarget.value);
    };
    return (React.createElement("div", { className: styles$7.wrapper, role: "search" },
        React.createElement("label", { htmlFor: id },
            React.createElement(Icon, { className: styles$7.icon, color: "neutral-gray", name: "search", size: 12 }),
            React.createElement("input", { defaultValue: value, id: id, onChange: handleChange, placeholder: placeholder, ref: ref, type: "search" }))));
};
SearchInputWithRef.displayName = 'SearchInput';
var SearchInput = React.forwardRef(SearchInputWithRef);

var styles$6 = {"creatableFooterContainer":"AddOptionsFooter-module__3lpPIsIg__v6-1-18","inputWrapper":"AddOptionsFooter-module__2h7N5O7p__v6-1-18","colorPicker":"AddOptionsFooter-module__2tQh2gdt__v6-1-18"};

var deriveFooterButtonState = function (inputValue, loading) {
    if (inputValue.length === 0)
        return 'disabled';
    if (loading)
        return 'loading';
    return 'enabled';
};
function AddOptionsFooter(_a) {
    var createButtonProps = _a.createButtonProps, inputProps = _a.inputProps, onCreate = _a.onCreate, colorPickerOptions = _a.colorPickerOptions;
    var _b = useState(''), inputValue = _b[0], setInputValue = _b[1];
    var _c = useState(colorPickerOptions === null || colorPickerOptions === void 0 ? void 0 : colorPickerOptions.initialColor), color = _c[0], setColor = _c[1];
    var onAddHandler = function () {
        onCreate(__assign({ value: inputValue || '' }, (color ? { color: color } : {})));
    };
    var onInputChange = function (event) {
        setInputValue(event.target.value);
    };
    return (React.createElement("footer", { className: styles$6.creatableFooterContainer },
        React.createElement(Input, { placeholder: inputProps === null || inputProps === void 0 ? void 0 : inputProps.placeholder, className: styles$6.inputWrapper, onKeyPress: function (e) {
                if (e.key.toLowerCase() === 'enter' && inputValue)
                    onAddHandler();
            }, metadata: inputProps === null || inputProps === void 0 ? void 0 : inputProps.metadata, onChange: onInputChange }),
        colorPickerOptions && (React.createElement("div", { className: styles$6.colorPicker },
            React.createElement(ColorPicker, { closeOnSelect: true, initialColor: color, metadata: colorPickerOptions.metadata, onChange: setColor, palette: colorPickerOptions.palette, title: colorPickerOptions.title }))),
        React.createElement(Button, { theme: "primary", onClick: onAddHandler, buttonState: deriveFooterButtonState(inputValue, createButtonProps.loading) }, createButtonProps.label)));
}

var styles$5 = {"loadingContainer":"LoadingState-module__1kM3jMVC__v6-1-18"};

function LoadingState() {
    return (React.createElement("div", { className: styles$5.loadingContainer, "aria-label": "loading" },
        React.createElement(Icon, { name: "spinner-third", size: 16, spin: true, color: "primary-brand-accent", type: "solid" })));
}

var styles$4 = {"submitFooterContainer":"SubmitFooter-module__1YkFWgco__v6-1-18","cancelButton":"SubmitFooter-module__IHUTMbpw__v6-1-18"};

var SubmitFooter = function (_a) {
    var onSubmit = _a.onSubmit, onCancel = _a.onCancel, _b = _a.submitLabel, submitLabel = _b === void 0 ? 'Apply' : _b, _c = _a.cancelSubmitLabel, cancelSubmitLabel = _c === void 0 ? 'Cancel' : _c;
    return (React.createElement("footer", { className: styles$4.submitFooterContainer },
        React.createElement(Typography, { component: "button", variant: "button-1", className: styles$4.cancelButton, onClick: onCancel, type: "button" }, cancelSubmitLabel),
        React.createElement(Button, { theme: "primary", onClick: onSubmit }, submitLabel)));
};

var styles$3 = {"selectAllOption":"SelectAllOption-module__Nph5J00o__v6-1-18"};

var SelectAllOption = function (_a) {
    var isSelected = _a.isSelected, onSelect = _a.onSelect, label = _a.label;
    var selectAllOption = useMemo(function () { return ({
        value: Date.now(),
        label: label,
    }); }, [label]);
    return (React.createElement(Option, { highlightedText: "", onSelect: function () { return onSelect(!isSelected); }, option: selectAllOption, optionStyle: "checkbox", optionType: "single-line", selected: isSelected, className: styles$3.selectAllOption }));
};

var MIN_OPTIONS_TO_DISPLAY_SEARCH = 7.5;
var MIN_OPTIONS_TO_DISPLAY_SELECT_ALL = 7.5;
var MIN_GROUPS_TO_DISPLAY_HEADER = 2;
function List(_a, ref) {
    var _b;
    var className = _a.className, selectAllLabel = _a.selectAllLabel, highlightedText = _a.highlightedText, variant = _a.variant, multiple = _a.multiple, onSelect = _a.onSelect, onSelectAll = _a.onSelectAll, onSearchChange = _a.onSearchChange, options = _a.options, optionType = _a.optionType, renderEmptyState = _a.renderEmptyState, searchable = _a.searchable, searchInitialValue = _a.searchInitialValue, searchAutofocus = _a.searchAutofocus, searchPlaceholder = _a.searchPlaceholder, searchNoResultsLabel = _a.searchNoResultsLabel, selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly, creatableOptions = _a.creatableOptions, style = _a.style, isLoading = _a.isLoading, submitOptions = _a.submitOptions, onSubmit = _a.onSubmit, onCancel = _a.onCancel, metadata = _a.metadata, restProps = __rest(_a, ["className", "selectAllLabel", "highlightedText", "variant", "multiple", "onSelect", "onSelectAll", "onSearchChange", "options", "optionType", "renderEmptyState", "searchable", "searchInitialValue", "searchAutofocus", "searchPlaceholder", "searchNoResultsLabel", "selectedOptionsWithValuesOnly", "creatableOptions", "style", "isLoading", "submitOptions", "onSubmit", "onCancel", "metadata"]);
    var _c = useState(searchInitialValue), searchInputValue = _c[0], setSearchInputValue = _c[1];
    var searchInputRef = useRef(null);
    useEffect(function () {
        if (!searchAutofocus || !searchInputRef.current)
            return;
        setTimeout(function () {
            if (searchInputRef.current)
                searchInputRef.current.focus();
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    var shouldShowSelectAll = multiple &&
        options.length > MIN_OPTIONS_TO_DISPLAY_SELECT_ALL &&
        searchInputValue.length === 0;
    var selectAllIsSelected = options.length === selectedOptionsWithValuesOnly.length;
    var filteredOptions = options.filter(function (option) { return searchSubstring(option.label, searchInputValue) !== -1; });
    var shouldShowNoResultsLabel = searchInputValue !== '' && filteredOptions.length === 0;
    var shouldShowSearchInput = searchable && options.length > MIN_OPTIONS_TO_DISPLAY_SEARCH;
    var groupedOptions = groupBy(filteredOptions, function (option) { return option.group; });
    var shouldShowGroups = Object.keys(groupedOptions).length >= MIN_GROUPS_TO_DISPLAY_HEADER;
    var renderOptions = function (optionsList) {
        var optionsRendered = optionsList.map(function (option) { return (React.createElement(Option, { disabled: option.disabled, highlightedText: searchable ? searchInputValue : highlightedText, key: option.value, onSelect: function () { return onSelect(option); }, option: option, optionStyle: multiple ? 'checkbox' : 'checkmark', optionType: optionType, selected: selectedOptionsWithValuesOnly.includes(option.value) })); });
        return (React.createElement(React.Fragment, null,
            shouldShowSelectAll && !shouldShowGroups && (React.createElement(SelectAllOption, { onSelect: onSelectAll, isSelected: selectAllIsSelected, label: selectAllLabel })),
            optionsRendered));
    };
    var renderOptionGroups = function () {
        var groupsRendered = toPairs(groupedOptions).map(function (_a) {
            var groupName = _a[0], groupOptions = _a[1];
            return (React.createElement(OptionGroup, { key: groupName, name: groupName }, renderOptions(groupOptions)));
        });
        return (React.createElement(React.Fragment, null,
            shouldShowSelectAll && (React.createElement(SelectAllOption, { onSelect: onSelectAll, isSelected: selectAllIsSelected, label: selectAllLabel })),
            groupsRendered));
    };
    var handleSearchInputChange = function (search) {
        setSearchInputValue(search);
        onSearchChange(search);
    };
    return (React.createElement("div", __assign({ ref: ref, className: classnames(styles$d.list, className, (_b = {},
            _b[styles$d.multiLineList] = optionType === 'multi-line',
            _b[styles$d.fullWidth] = variant === 'full-width',
            _b)), style: style }, parseMetadata(suffixMetadata(metadata, 'list')), restProps), isLoading ? (React.createElement(LoadingState, null)) : (React.createElement(React.Fragment, null,
        shouldShowSearchInput && (React.createElement(SearchInput, { ref: searchInputRef, placeholder: searchPlaceholder, value: searchInputValue, onTextChange: handleSearchInputChange })),
        shouldShowNoResultsLabel && (React.createElement(Typography, { component: "div", variant: "body-1", className: styles$d.noResults }, searchNoResultsLabel + " \"" + searchInputValue + "\"")),
        !shouldShowNoResultsLabel &&
            (options.length === 0 ? (renderEmptyState()) : (React.createElement("ul", { role: "listbox" }, shouldShowGroups
                ? renderOptionGroups()
                : renderOptions(filteredOptions)))),
        creatableOptions && (React.createElement(AddOptionsFooter, { onCreate: creatableOptions.onCreate, createButtonProps: creatableOptions.createButtonProps, inputProps: creatableOptions.inputProps, colorPickerOptions: creatableOptions.colorPickerOptions })),
        !creatableOptions && submitOptions && (React.createElement(SubmitFooter, { onSubmit: onSubmit, onCancel: onCancel, submitLabel: submitOptions.submitLabel, cancelSubmitLabel: submitOptions.cancelLabel }))))));
}
var ListWithRef = forwardRef(List);

var styles$2 = {"defaultEmptyState":"DefaultEmptyState-module__2GMCdSkP__v6-1-18"};

function DefaultEmptyState(_a) {
    var _b = _a === void 0 ? { label: 'Nothing to display' } : _a, label = _b.label;
    return (React.createElement(Typography, { className: styles$2.defaultEmptyState, variant: "body-1", component: "div" }, label));
}

var styles$1 = {"selectTrigger":"DefaultTrigger-module__1Lcj4UJj__v6-1-18","selected":"DefaultTrigger-module__2pDPCVHO__v6-1-18","fullWidth":"DefaultTrigger-module__16yCD6zm__v6-1-18","innerText":"DefaultTrigger-module__1VgmRcUO__v6-1-18","outline-warning":"DefaultTrigger-module__3iRFUG-L__v6-1-18","outline-error":"DefaultTrigger-module__1zBxYiiH__v6-1-18","disabled":"DefaultTrigger-module__e9ZX9U5___v6-1-18","icon":"DefaultTrigger-module__3O-cbhK6__v6-1-18"};

var styles = {"selectClearButton":"SelectClearButton-module__3m8ZA7Fl__v6-1-18","clearIcon":"SelectClearButton-module__3M_pe9ou__v6-1-18"};

var SelectClearButton = function (_a) {
    var onClick = _a.onClick, disabled = _a.disabled, _b = _a.ariaLabelClearButton, ariaLabelClearButton = _b === void 0 ? 'Clear Select Input' : _b;
    return (React.createElement("div", { role: "button", className: styles.selectClearButton, onClick: disabled ? noop$1 : onClick, onKeyPress: disabled ? noop$1 : onClick, tabIndex: -1, "aria-label": ariaLabelClearButton },
        React.createElement(Icon, { name: "times-circle", className: styles.clearIcon, type: "solid" })));
};

function DefaultTrigger(_a) {
    var _b;
    var disabled = _a.disabled, variant = _a.variant, label = _a.label, multiple = _a.multiple, onClick = _a.onClick, selectedOptions = _a.selectedOptions, visible = _a.visible, _c = _a.outline, outline = _c === void 0 ? 'default' : _c, _d = _a.autoFocus, autoFocus = _d === void 0 ? false : _d, triggerId = _a.triggerId, onClear = _a.onClear, ref = _a.ref;
    var hasSelectedOption = selectedOptions.length > 0;
    var renderSingleSelectionLabel = function () {
        var firstItem = selectedOptions[0];
        return hasSelectedOption ? firstItem.label : label;
    };
    var renderMultiSelectionLabel = function () {
        return hasSelectedOption ? label + " \u00B7 " + selectedOptions.length : label;
    };
    var isFullWidth = variant === 'full-width';
    return (React.createElement("button", __assign({ ref: ref, className: classnames(styles$1.selectTrigger, styles$1["outline-" + outline], (_b = {},
            _b[styles$1.fullWidth] = isFullWidth,
            _b[styles$1.selected] = hasSelectedOption,
            _b[styles$1.disabled] = disabled,
            _b)), type: "button", disabled: disabled, onClick: onClick, 
        // eslint-disable-next-line jsx-a11y/no-autofocus
        autoFocus: autoFocus }, (triggerId ? { id: triggerId } : {})),
        React.createElement("span", { className: styles$1.innerText }, multiple ? renderMultiSelectionLabel() : renderSingleSelectionLabel()),
        onClear && selectedOptions.length > 0 ? (React.createElement(SelectClearButton, { onClick: onClear, disabled: disabled })) : null,
        React.createElement(Icon, { className: styles$1.icon, name: visible ? 'angle-up' : 'angle-down', type: "solid" })));
}

var useListVisibilityChangeHandler = function (onListVisibilityChange, visible) {
    var initialRender = useRef(true);
    useEffect(function () {
        if (initialRender.current) {
            initialRender.current = false;
        }
        else {
            onListVisibilityChange(visible);
        }
    }, [onListVisibilityChange, visible]);
};
function SelectWithRef(_a, ref) {
    var className = _a.className, listClassName = _a.listClassName, _b = _a.closeOnSelect, closeOnSelect = _b === void 0 ? false : _b, _c = _a.disabled, disabled = _c === void 0 ? false : _c, _d = _a.variant, variant = _d === void 0 ? 'inline' : _d, _e = _a.selectAllLabel, selectAllLabel = _e === void 0 ? 'Select all' : _e, metadata = _a.metadata, _f = _a.multiple, multiple = _f === void 0 ? false : _f, _g = _a.onSearchChange, onSearchChange = _g === void 0 ? noop$1 : _g, _h = _a.onSelect, onSelect = _h === void 0 ? noop$1 : _h, _j = _a.onListVisibilityChange, onListVisibilityChange = _j === void 0 ? noop$1 : _j, options = _a.options, _k = _a.renderEmptyState, renderEmptyState = _k === void 0 ? DefaultEmptyState : _k, _l = _a.renderTrigger, renderTrigger = _l === void 0 ? DefaultTrigger : _l, _m = _a.optionType, optionType = _m === void 0 ? 'single-line' : _m, _o = _a.searchable, searchable = _o === void 0 ? true : _o, _p = _a.searchAutofocus, searchAutofocus = _p === void 0 ? true : _p, _q = _a.searchInitialValue, searchInitialValue = _q === void 0 ? '' : _q, _r = _a.searchNoResultsLabel, searchNoResultsLabel = _r === void 0 ? 'No results for' : _r, _s = _a.searchPlaceholder, searchPlaceholder = _s === void 0 ? 'Search' : _s, _t = _a.selectedOptions, selectedOptions = _t === void 0 ? [] : _t, _u = _a.triggerLabel, triggerLabel = _u === void 0 ? '' : _u, _v = _a.outline, outline = _v === void 0 ? 'default' : _v, _w = _a.autoFocus, autoFocus = _w === void 0 ? false : _w, triggerId = _a.triggerId, creatableOptions = _a.creatableOptions, _x = _a.isClearable, isClearable = _x === void 0 ? false : _x, _y = _a.highlightedText, highlightedText = _y === void 0 ? '' : _y, _z = _a.isLoading, isLoading = _z === void 0 ? false : _z, submitOptions = _a.submitOptions;
    var _0 = build.exports.useClickOutside(), clickOutsideRef = _0[0], hasClickedOutside = _0[1];
    var _1 = useState(false), visible = _1[0], setVisibility = _1[1];
    var _2 = useAutoPositioner(), setReferenceElement = _2.setReferenceElement, setPopperElement = _2.setPopperElement, popperStyles = _2.styles.popper, attributes = _2.attributes;
    useListVisibilityChangeHandler(onListVisibilityChange, visible);
    useEffect(function () {
        if (hasClickedOutside && visible) {
            setVisibility(false);
            if (submitOptions) {
                submitOptions.onCancel();
            }
        }
    }, [hasClickedOutside, visible, submitOptions]);
    var selectedOptionsWithValuesOnly = useMemo(function () {
        return selectedOptions.map(function (selectedOption) { return selectedOption.value; });
    }, [selectedOptions]);
    var beforeSelect = function (callback) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (closeOnSelect)
                setVisibility(false);
            callback.apply(void 0, args);
        };
    };
    var handleClear = function (evt) {
        evt.stopPropagation();
        var fixedOptions = getFixedOptions(options, selectedOptions);
        onSelect(fixedOptions);
        if (submitOptions) {
            submitOptions.onSubmit(fixedOptions);
        }
        setVisibility(false);
    };
    var handleSelectAll = function (selected) {
        var opts = selected
            ? getAllSelectableOptions(options, selectedOptions)
            : getFixedOptions(options, selectedOptions);
        onSelect(opts);
    };
    var handleSelect = function (option) {
        onSelect(getSelectedValues({
            option: option,
            selectedOptions: selectedOptions,
            selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly,
            multiple: multiple,
        }));
    };
    var handleTriggerClick = function () {
        setVisibility(function (currentVisibility) { return !currentVisibility; });
        if (visible && submitOptions) {
            submitOptions.onCancel();
        }
    };
    var finalRef = useForkRef([clickOutsideRef, ref]);
    var _3 = useState(), triggerWidth = _3[0], setTriggerWidth = _3[1];
    var triggerRef = useForkRef([
        setReferenceElement,
        function (elem) {
            if (elem)
                setTriggerWidth(getComputedStyle(elem).width);
        },
    ]);
    var handleSubmit = function () {
        if (submitOptions)
            submitOptions.onSubmit(selectedOptions);
        setVisibility(false);
    };
    var handleCancelSubmit = function () {
        setVisibility(false);
        if (submitOptions)
            submitOptions.onCancel();
    };
    return (React.createElement("div", __assign({ className: className, ref: finalRef }, parseMetadata(metadata)),
        React.createElement("div", null, renderTrigger({
            selectRef: clickOutsideRef,
            ref: triggerRef,
            disabled: disabled,
            variant: variant,
            label: triggerLabel,
            multiple: multiple,
            onClick: handleTriggerClick,
            selectedOptions: selectedOptions,
            visible: visible,
            outline: outline,
            autoFocus: autoFocus,
            triggerId: triggerId,
            setListVisibility: setVisibility,
            onClear: isClearable ? handleClear : undefined,
        })),
        visible && (React.createElement(ListWithRef, __assign({ className: listClassName, metadata: metadata, selectAllLabel: selectAllLabel, isLoading: isLoading, highlightedText: highlightedText, variant: variant, multiple: multiple, onSelect: beforeSelect(handleSelect), onSelectAll: beforeSelect(handleSelectAll), onSearchChange: onSearchChange, options: options, optionType: optionType, renderEmptyState: renderEmptyState, selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly, searchable: searchable, searchNoResultsLabel: searchNoResultsLabel, searchPlaceholder: searchPlaceholder, searchInitialValue: searchInitialValue, searchAutofocus: searchAutofocus, onSubmit: handleSubmit, onCancel: handleCancelSubmit, submitOptions: submitOptions, creatableOptions: creatableOptions
                ? __assign(__assign({}, creatableOptions), { onCreate: function (createdItem) {
                        if (closeOnSelect && !multiple)
                            setVisibility(false);
                        creatableOptions.onCreate(createdItem);
                    } }) : creatableOptions, ref: setPopperElement, style: __assign(__assign(__assign({}, popperStyles), (variant === 'full-width' ? { width: triggerWidth } : {})), (isLoading ? { minWidth: triggerWidth } : {})) }, attributes.popper)))));
}
SelectWithRef.displayName = 'Select';
/**
 * According to this article: https://fettblog.eu/typescript-react-generic-forward-refs/
 */
var Select = React.forwardRef(SelectWithRef);

export { Select };
//# sourceMappingURL=index.js.map
